"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const events_1 = require("events");
class PromiseQueue extends events_1.EventEmitter {
    constructor(config) {
        super();
        this._running = 0;
        this._hasError = false;
        if (!config)
            throw new Error('Missing config parameter');
        if (!config.promises && (!config.items || !config.promise))
            throw new Error('Missing items');
        if (!config.concurrence)
            throw new Error('Missing concurrence');
        this._items = Array.from(config.items || []);
        this._promise = config.promise;
        this._promises = config.promises;
        this._concurrence = config.concurrence;
        this._shouldStopOnError = !!config.shouldStopOnError;
        this._setupListeners();
    }
    start() {
        const canRun = this._canRun();
        if (!canRun)
            return;
        const queueSize = this._items ? this._items.length : this._promises.length;
        const firstInteractionSize = Math.min(queueSize, this._concurrence);
        for (let i = 0; i < firstInteractionSize; i += 1) {
            this._runNextPromise();
        }
    }
    cancel() {
        this._concurrence = 0;
    }
    _setupListeners() {
        this.on(PromiseQueue.PRIVATE_EVENTS.ERROR, (errorItem) => {
            this._hasError = true;
            this.emit(PromiseQueue.EVENTS.ITEM_ERROR, errorItem);
        });
        this.on(PromiseQueue.PRIVATE_EVENTS.PROCESSED, () => {
            this._running -= 1;
            const canRun = this._canRun();
            if (!canRun)
                return;
            this._runNextPromise();
        });
    }
    _executePromise(promise, item) {
        promise(item)
            .then((res) => {
            this.emit(PromiseQueue.EVENTS.ITEM_PROCESSED, { res, item });
        })
            .catch((err) => {
            this.emit(PromiseQueue.PRIVATE_EVENTS.ERROR, { err, item });
        })
            .finally(() => {
            this.emit(PromiseQueue.PRIVATE_EVENTS.PROCESSED);
        });
        this.emit(PromiseQueue.EVENTS.ITEM_PROCESSING, { item });
    }
    _runNextPromise() {
        if (this._promises) {
            const promise = this._promises.shift();
            this._executePromise(promise);
        }
        else {
            const item = this._items.shift();
            this._executePromise(this._promise, item);
        }
        this._running += 1;
    }
    _canRun() {
        if (this._shouldStopOnError && this._hasError) {
            if (this._running === 0) {
                this.emit(PromiseQueue.EVENTS.QUEUE_PROCESSED, this._hasError);
            }
            else {
                this.cancel();
            }
            return false;
        }
        const hasWaitingItems = (this._items && this._items.length > 0)
            || (this._promises && this._promises.length > 0);
        if (!hasWaitingItems && this._running === 0) {
            this.emit(PromiseQueue.EVENTS.QUEUE_PROCESSED, this._hasError);
            return false;
        }
        const canExecuteNextPromise = hasWaitingItems && this._concurrence > this._running;
        return canExecuteNextPromise;
    }
}
exports.default = PromiseQueue;
PromiseQueue.EVENTS = {
    ITEM_ERROR: 'item_error',
    ITEM_PROCESSED: 'item_processed',
    ITEM_PROCESSING: 'item_processing',
    QUEUE_PROCESSED: 'queue_processed',
};
PromiseQueue.PRIVATE_EVENTS = {
    ERROR: 'error',
    PROCESSED: 'processed',
};
